/*************************************
 MapView 1.1
 (C) 2004-2007 Hynek Sladky
 sladky@mujmail.cz
*************************************/
// ------------------------------------
// code section definition
// ------------------------------------
#define SECT_FUNC  __attribute__ ((__section__("code_fun")))
//#define SECT_FUNC
// ------------------------------------
// database type definition
// ------------------------------------
#define AppID           'BMvw'
#define HistID          'Hist'
#define POI_ID          'POI_'
// ------------------------------------
// user Buttons
// ------------------------------------
enum {
  btnDateBook,
  btnDateBookLong,
  btnAddress,
  btnAddressLong,
  btnToDo,
  btnToDoLong,
  btnMemopad,
  btnMemopadLong,
  btnPageUp,
  btnPageUpLong,
  btnPageDown,
  btnPageDownLong,
  btn5wayLeft,
  btn5wayLeftLong,
  btn5wayRight,
  btn5wayRightLong,
  btn5waySelect,
  btn5waySelectLong,
  btnJogUp,
  btnJogUpLong,
  btnJogDown,
  btnJogDownLong,
  btnJogPush,
  btnJogPushLong,
  btnJogBack,
  btnJogBackLong,
  btn_count
};
// ------------------------------------
// command bar icon IDs
// ------------------------------------
typedef const struct {
  UInt16 bitmap;
  UInt32 action;
} BarIconType;
// ------------------------------------
// user Button actions
// ------------------------------------
enum {
  mapMoveUp=firstUserEvent,
  mapMoveDown,
  mapMoveLeft,
  mapMoveRight,
  mapZoomIn,
  mapZoomOut,
  mapOpen,
  mapPOI,
  mapGrid,
  mapRotateL,
  mapRotateR,
  mapHistory,
  mapSetPoint1,
  mapAddPoint2,
  mapOverview,
  mapDistances,
  mapInfo,
  sysBackLight,
  sysMenu,
  // internal actions
  mapMaxKeyEvent,
  mapCenter,
  mapTop
};
// ------------------------------------
// Preferences space
// ------------------------------------
enum {
  homedir,
  pref_dummy_str_1, // homemap was here but is no longer supported
  overviewmap,
  coordmap,
  gridunit,
  prefstrcnt
};
// ------------------------------------
// Preference flags
// ------------------------------------
#define CartoonMode     0x0001
#define	Root_only	0x0004
#define ColMask         0x0300
#define ColBits         8
#define RotMask         0x0C00
#define RotBits         10
// ------------------------------------
// program state
// ------------------------------------
#define Grid_mask       0x00010000
#define GridInside      0x00020000
#define RedrawMap       0x00040000
// ------------------------------------
// ------------------------------------
#define dirUp           0x80
#define dirLeft         0x40
#define dirDown         0x20
#define dirRight        0x10

#define dirCartLine     0x02
#define dirCartPage     0x04
// ------------------------------------
// point entry variable
// ------------------------------------
typedef enum {pointNone, point_POI, point1, point2, point_AzDi} selPointType;
extern selPointType selectPoint;
// ------------------------------------
// preference type
// ------------------------------------
typedef struct {
  struct {
    UInt32        flags;
//    Int32         homeX, homeY;
    Int32         coord1X, coord1Y;
    Int32         coord2X, coord2Y;
    RGBColorType  gridCol;
    UInt16        buttons[btn_count];
    double        distance, precision;
    UInt32        cmd_icons;
  } bin;
  Char            str[prefstrcnt][256];
} PrefsType;
// ------------------------------------
// History structure
// ------------------------------------
typedef struct {
  Int32         startX, startY;
  UInt16        volRefNum;
  Char          name[256];
} historyType;
// ------------------------------------
// History Database
// ------------------------------------
#define     MaxHistoryEntries    13
// ------------------------------------
// LCD variables
// ------------------------------------
typedef struct {
  // basic information
  UInt32        modes, bits; // allowed modes, actual bpp
  struct {
    struct {
      UInt32    width, height;
    } native, standard;
  } device, form, actual;
  UInt32        density;
  UInt8         translate, rotate;
  Boolean       systemPalette, resize;
  // Sony HiRes variables
  UInt16        sonyHiResRefNum;
  // conversion from 8bpp user palette to 8bpp system palette
  UInt8         white, black, grid;
  UInt8         ColorTable[256];
  UInt8         GridTable[256];
  // conversion from 8bpp to 16bpp direct color
  UInt16        RGB16[256];
} LCDinfoType;
// ------------------------------------
// VFS directory entries
// ------------------------------------
#define MaxDIRentries   256
#define LinkFlag        0x01
#define	root_map	0x80
typedef struct {
  UInt16        volRefNum;
  UInt16        zoom;
  UInt8         index;
  UInt8         namelen;
  UInt8         flags;
  Char          name[1];
} direntry;
typedef struct {
  Char          filename[256];
  FileRef       fileref;
  UInt8         index;
} cacheentry;
typedef struct {
  // global variables for directory enumeration
  UInt32 volIterator, dirIterator;
  FileRef DIRref;
  UInt16 volRefNum;
  // 'Open map' variables, map directory list variables
  Char          **openlist;
  UInt16	mapcount, opencount;
  UInt8         DirectoryOK;
  direntry   	**maplist;
  UInt16        linkindex;
} VFSinfoType;
// ------------------------------------
// map file information
// ------------------------------------
#define     CartoonFile      0x0040
#define     CompressedFile   0x0080
#define     CompressedMask   0x0083
#define     CompressedZLIB   0x0080
#define     CompressedLZO    0x0081
#define     CompressedMax    0x0081
/******** 
  version description
- HiByte is file version
  0x00
  0x01
  0x02
- LoByte contains file flags
  0x80 - compression
  0x80 - Zlib compression
  0x81 - LZO compression
  0x40 - cartoon mode
********/
// ------------------------------------
// map info structures
// ------------------------------------
typedef struct {
  UInt16 id;
  UInt32 size;
} add_hdr_type;
typedef struct {
  UInt32 pixels;
  UInt32 measure;
  Char unit[32];
} grid_type;
typedef struct {
  UInt32 Nx_offs, Ny_offs;
  UInt32 Nx, Ny;
  Char name_mask[256];
} SupTileType;
enum {
  gridX_id=0x0001,
  gridY_id,
  gridIN_id,
  SupTile,
  GPS_info
};
typedef struct {
  char signature[4];
  UInt16 version;
  UInt32 size_x, size_y; // pixel size
  UInt32 Nx, Ny; // tile count
  UInt16 Wx, Wy; // tile size
  UInt32 min_x, min_y, max_x, max_y;
  UInt32 table_offs; // color table
  UInt32 list_offs; // offset list of tiles
  UInt32 add_offs; // additional informations in version 0x02
  // new properties should be added here to retain compatibility...
} MapHeaderType;
// ------------------------------------
// GPS calibration values
// ------------------------------------
typedef struct {
  //double rel_x, rel_y;
  //double GPS_x, GPS_y;
  FlpCompDouble x, y;
  FlpCompDouble lat, lon; //GPS_y, GPS_x; // latitude, longitude
} GPSinfoType;
// ------------------------------------
// actual map information
// ------------------------------------
typedef struct {
  // map header v1
  MapHeaderType MapHeader;
  Boolean valid;
  // filename and volume number for main map file, position in map
  historyType   curr_map;
  // filename w/ path and extension and mask for multi-file maps
  Char          curr_filename[256];
  // LCD size in tiles
  UInt16        Xtiles, Ytiles;
  // partial tile offsets
  Int32         xOff, yOff, xRest, yRest;
  // relative position in map for inter-map calculations
  double        rel_pos_x, rel_pos_y;
  // performance analysis variables
  UInt16        ratiocount;
  UInt32	speed_index, ratio1, ratio2;
  // additional map information
  grid_type     gridX, gridY;
  SupTileType   SuperTile;
  UInt16        namelen;
  UInt16        GPScount;
  GPSinfoType   *GPScal;
} MapInfoType;
// ------------------------------------
// drawing space variables
// ------------------------------------
typedef struct {
  // ZLib variables
  UInt16	ZLibRefNum;
  // map loading and drawing space variables
  UInt8         *zip_data;
  BitmapType    *bitmap, *oldbmp;
  UInt8         *bitdata;
} DrawInfoType;
// ------------------------------------
// palette structure definition
// ------------------------------------
typedef union {
  struct {
    unsigned char B, G, R;
  } rgb;
  unsigned int dword;
} colordef;
// ------------------------------------
// PointOfInterest variables
// ------------------------------------
typedef struct {
  historyType   fileinfo;
  Char          name[256];
} POItype;
// ====================================
// global variable references
// ------------------------------------
extern DrawInfoType DrawInfo;
extern MapInfoType MapInfo;
extern LCDinfoType LCD;
extern PrefsType Prefs;
extern VFSinfoType VFSInfo;
// ====================================
// function prototypes
// ------------------------------------
void GetDirEntry (UInt16 cnt);
Int16 SearchMap (Char *str);
void CalcRotatedVars (void);
void SetLCDmode (UInt8 bpp);
void DEBUG (char *a, ...);
void CheckStartXY (void);

void RotateDirRight (Int16 *dir);
void DistanceWindow (Boolean save);
void RotateDirLeft (Int16 *dir);

